@@ -1,5 +1,3 @@
-# -*- coding:utf-8 -*-
-
 """
 Simple static page generator.
 
@@ -9,6 +7,7 @@ Uses Jinja2 to compile templates.
 import inspect
 import logging
 import os
+from pathlib import Path
 import re
 import shutil
 import warnings
@@ -18,16 +17,23 @@ from jinja2 import Environment, FileSystemLoader
 from .reloader import Reloader
 
 
-def _has_argument(func):
-    """Test whether a function expects an argument.
-
-    :param func:
-        The function to be tested for existence of an argument.
-    """
-    return bool(inspect.signature(func).parameters)
+def _compute_context(context_like, template):
+    if callable(context_like):
+        has_argument = bool(inspect.signature(context_like).parameters)
+        if has_argument:
+            return context_like(template)
+        else:
+            return context_like()
+    else:
+        return context_like
 
 
-class Site(object):
+def _ensure_dir(path):
+    """Ensure the directory for a file exists."""
+    os.makedirs(os.path.dirname(Path(path)), exist_ok=True)
+
+
+class Site:
     """The Site object.
 
     :param environment:
@@ -183,7 +189,8 @@ class Site(object):
         # Coerce search to an absolute path if it is not already
         if not os.path.isabs(searchpath):
             # TODO: Determine if there is a better way to do this
-            calling_module = inspect.getmodule(inspect.stack()[-1][0])
+            last_frame_info = inspect.stack()[-1]
+            calling_module = inspect.getmodule(last_frame_info.frame)
             if calling_module is None:
                 # Called from the interpreter or similar
                 project_path = os.getcwd()
@@ -234,6 +241,7 @@ class Site(object):
 
         :param template_name: A string representing the name of the template.
         """
+        template_name = Path(template_name).as_posix()
         try:
             return self.env.get_template(template_name)
         except UnicodeDecodeError as e:
@@ -254,16 +262,10 @@ class Site(object):
         :param template: the template to get the context for
         """
         context = {}
-        for regex, context_generator in self.contexts:
+        for regex, context_like in self.contexts:
             if re.match(regex, template.name):
-                if inspect.isfunction(context_generator):
-                    if _has_argument(context_generator):
-                        context.update(context_generator(template))
-                    else:
-                        context.update(context_generator())
-                else:
-                    context.update(context_generator)
-
+                new_context = _compute_context(context_like, template)
+                context.update(new_context)
                 if not self.mergecontexts:
                     break
         return context
@@ -280,41 +282,42 @@ class Site(object):
                 return render_func
         raise ValueError("no matching rule")
 
-    def is_static(self, template_name):
-        """Check if a template is a static template. Static template are copied,
-        rather than compiled using Jinja2.
+    def is_static(self, filename):
+        """Check if a file is static. Static files are copied, rather than
+        compiled using Jinja2.
 
         .. deprecated:: 0.3.4
 
         A template is considered static if it lives in any of the directories
         specified in ``staticpaths``.
 
-        :param template_name: the name of the template to check
+        :param filename: A PathLike name of the file to check.
 
         """
-        return any(template_name.startswith(path) for path in self.staticpaths)
+        path = Path(filename).as_posix()
+        return any(path.startswith(Path(sp).as_posix()) for sp in self.staticpaths)
 
-    def is_partial(self, template_name):
-        """Check if a template is a partial template. Partial templates are not
+    def is_partial(self, filename):
+        """Check if a file is partial. Partial files are not
         rendered, but they are used in rendering templates.
 
-        A template is considered a partial if it or any of its parent
+        A file is considered a partial if it or any of its parent
         directories are prefixed with an ``'_'``.
 
-        :param template_name: the name of the template to check
+        :param filename: A PathLike name of the file to check
         """
-        return any((x.startswith("_") for x in template_name.split("/")))
+        return any(part.startswith("_") for part in Path(filename).parts)
 
-    def is_ignored(self, template_name):
-        """Check if a template is an ignored template. Ignored templates are
+    def is_ignored(self, filename):
+        """Check if a file is an ignored. Ignored files are
         neither rendered nor used in rendering templates.
 
-        A template is considered ignored if it or any of its parent directories
+        A file is considered ignored if it or any of its parent directories
         are prefixed with an ``'.'``.
 
-        :param template_name: the name of the template to check
+        :param filename: A PathLike name of the file to check
         """
-        return any((x.startswith(".") for x in template_name.split("/")))
+        return any(part.startswith(".") for part in Path(filename).parts)
 
     def is_template(self, filename):
         """Check if a file is a template.
@@ -322,7 +325,7 @@ class Site(object):
         A file is a considered a template if it is not partial, ignored, or
         static.
 
-        :param filename: the name of the file to check
+        :param filename: A PathLike name of the file to check
         """
         if self.is_partial(filename):
             return False
@@ -332,14 +335,6 @@ class Site(object):
             return False
         return True
 
-    def _ensure_dir(self, template_name):
-        """Ensure the output directory for a template exists."""
-        head = os.path.dirname(template_name)
-        if head:
-            file_dirpath = os.path.join(self.outpath, head)
-            if not os.path.exists(file_dirpath):
-                os.makedirs(file_dirpath)
-
     def render_template(self, template, context=None, filepath=None):
         """Render a single :class:`jinja2.Template` object.
 
@@ -355,25 +350,20 @@ class Site(object):
             used to provide a context.
 
         :param filepath:
-            Optional. A file or file-like object to dump the complete template
-            stream into. Defaults to to ``os.path.join(self.outpath,
-            template.name)``.
-
+            Optional. A PathLike representing the output location.
+            Defaults to to ``os.path.join(self.outpath, template.name)``.
         """
         self.logger.info("Rendering %s..." % template.name)
 
         if context is None:
             context = self.get_context(template)
 
-        if not os.path.exists(self.outpath):
-            os.makedirs(self.outpath)
-        self._ensure_dir(template.name)
-
         try:
             rule = self.get_rule(template.name)
         except ValueError:
             if filepath is None:
                 filepath = os.path.join(self.outpath, template.name)
+            _ensure_dir(filepath)
             template.stream(**context).dump(filepath, self.encoding)
         else:
             rule(self, template, **context)
@@ -389,28 +379,48 @@ class Site(object):
 
     def copy_static(self, files):
         for f in files:
-            input_location = os.path.join(self.searchpath, f)
-            output_location = os.path.join(self.outpath, f)
-            self.logger.info("Copying %s to %s." % (f, output_location))
-            self._ensure_dir(f)
+            f = Path(f)
+            input_location = Path(self.searchpath) / f
+            output_location = Path(self.outpath) / f
+            self.logger.info("Copying %s to %s.", f, output_location)
+            _ensure_dir(output_location)
             shutil.copy2(input_location, output_location)
 
-    def get_dependencies(self, filename):
-        """Get a list of files that depends on the file named *filename*.
+    def get_dependents(self, filename):
+        """Get a list of files that depends on *filename*. Useful to decide
+        what to re-render when *filename* changes.
 
-        :param filename: the name of the file to find dependencies of
+        - Ignored files have no dependents.
+        - Static and template files just have themselves as dependents.
+        - Partial files have all the templates as dependents, since any template
+          may rely upon a partial.
+
+        .. versionchanged:: 1.1.0
+           Now always returns list of filenames. Before the return type
+           was either a list of templates or list of filenames.
+
+        :param filename: the name of the file to find dependents of
+        :return: list of filenames of dependents.
         """
-        template_name = filename.replace(os.path.sep, "/")
-
-        if self.is_partial(template_name):
-            return self.templates
-        elif self.is_template(template_name):
-            return [self.get_template(filename)]
-        elif self.is_static(template_name):
+        if self.is_partial(filename):
+            return self.template_names
+        elif self.is_template(filename):
+            return [filename]
+        elif self.is_static(filename):
             return [filename]
         else:
             return []
 
+    def get_dependencies(self, filename):
+        """
+        .. deprecated:: 1.1.0
+           Use :meth:`Site.get_dependents` instead.
+        """
+        warnings.warn(
+            "Site.get_dependencies() is deprecated. Use Site.get_dependents() instead."
+        )
+        return self.get_dependents(filename)
+
     def render(self, use_reloader=False):
         """Generate the site.
 
@@ -420,8 +430,6 @@ class Site(object):
         self.copy_static(self.static_names)
 
         if use_reloader:
-            self.logger.info("Watching '%s' for changes..." % self.searchpath)
-            self.logger.info("Press Ctrl+C to stop.")
             Reloader(self).watch()
 
     def __repr__(self):
@@ -435,7 +443,7 @@ class Renderer(Site):
            Use :meth:`Site.make_site` instead.
         """
         warnings.warn("Renderer was renamed to Site.")
-        super(Renderer, Site).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def run(self, use_reloader=False):
         return self.render(use_reloader)
